# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kv_rpc.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import bookkeeper.proto.kv_pb2 as kv__pb2
import bookkeeper.proto.storage_pb2 as storage__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='kv_rpc.proto',
  package='bookkeeper.proto.kv.rpc',
  syntax='proto3',
  serialized_options=_b('\n)org.apache.bookkeeper.stream.proto.kv.rpcP\001'),
  serialized_pb=_b('\n\x0ckv_rpc.proto\x12\x17\x62ookkeeper.proto.kv.rpc\x1a\x08kv.proto\x1a\rstorage.proto\"C\n\rRoutingHeader\x12\x11\n\tstream_id\x18\x01 \x01(\x03\x12\x10\n\x08range_id\x18\x02 \x01(\x03\x12\r\n\x05r_key\x18\x03 \x01(\x0c\"\x84\x01\n\x0eResponseHeader\x12\x32\n\x04\x63ode\x18\x01 \x01(\x0e\x32$.bookkeeper.proto.storage.StatusCode\x12>\n\x0erouting_header\x18\x63 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RoutingHeader\"\xb2\x04\n\x0cRangeRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x11\n\trange_end\x18\x02 \x01(\x0c\x12\r\n\x05limit\x18\x03 \x01(\x03\x12\x10\n\x08revision\x18\x04 \x01(\x03\x12\x43\n\nsort_order\x18\x05 \x01(\x0e\x32/.bookkeeper.proto.kv.rpc.RangeRequest.SortOrder\x12\x45\n\x0bsort_target\x18\x06 \x01(\x0e\x32\x30.bookkeeper.proto.kv.rpc.RangeRequest.SortTarget\x12\x14\n\x0cserializable\x18\x07 \x01(\x08\x12\x11\n\tkeys_only\x18\x08 \x01(\x08\x12\x12\n\ncount_only\x18\t \x01(\x08\x12\x18\n\x10min_mod_revision\x18\n \x01(\x03\x12\x18\n\x10max_mod_revision\x18\x0b \x01(\x03\x12\x1b\n\x13min_create_revision\x18\x0c \x01(\x03\x12\x1b\n\x13max_create_revision\x18\r \x01(\x03\x12\x36\n\x06header\x18\x63 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RoutingHeader\".\n\tSortOrder\x12\x08\n\x04NONE\x10\x00\x12\n\n\x06\x41SCEND\x10\x01\x12\x0b\n\x07\x44\x45SCEND\x10\x02\"B\n\nSortTarget\x12\x07\n\x03KEY\x10\x00\x12\x0b\n\x07VERSION\x10\x01\x12\n\n\x06\x43REATE\x10\x02\x12\x07\n\x03MOD\x10\x03\x12\t\n\x05VALUE\x10\x04\"\x91\x01\n\rRangeResponse\x12\x37\n\x06header\x18\x01 \x01(\x0b\x32\'.bookkeeper.proto.kv.rpc.ResponseHeader\x12*\n\x03kvs\x18\x02 \x03(\x0b\x32\x1d.bookkeeper.proto.kv.KeyValue\x12\x0c\n\x04more\x18\x03 \x01(\x08\x12\r\n\x05\x63ount\x18\x04 \x01(\x03\"\xc6\x01\n\nPutRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\r\n\x05lease\x18\x03 \x01(\x03\x12\x0f\n\x07prev_kv\x18\x04 \x01(\x08\x12\x14\n\x0cignore_value\x18\x05 \x01(\x08\x12\x14\n\x0cignore_lease\x18\x06 \x01(\x08\x12\x36\n\x06header\x18\x63 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RoutingHeader\x12\x18\n\x10\x65xpected_version\x18\x64 \x01(\x03\"v\n\x0bPutResponse\x12\x37\n\x06header\x18\x01 \x01(\x0b\x32\'.bookkeeper.proto.kv.rpc.ResponseHeader\x12.\n\x07prev_kv\x18\x02 \x01(\x0b\x32\x1d.bookkeeper.proto.kv.KeyValue\"z\n\x10IncrementRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x03\x12\x11\n\tget_total\x18\x03 \x01(\x08\x12\x36\n\x06header\x18\x63 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RoutingHeader\"b\n\x11IncrementResponse\x12\x37\n\x06header\x18\x01 \x01(\x0b\x32\'.bookkeeper.proto.kv.rpc.ResponseHeader\x12\x14\n\x0ctotal_amount\x18\x02 \x01(\x03\"}\n\x12\x44\x65leteRangeRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x11\n\trange_end\x18\x02 \x01(\x0c\x12\x0f\n\x07prev_kv\x18\x03 \x01(\x08\x12\x36\n\x06header\x18\x63 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RoutingHeader\"\x90\x01\n\x13\x44\x65leteRangeResponse\x12\x37\n\x06header\x18\x01 \x01(\x0b\x32\'.bookkeeper.proto.kv.rpc.ResponseHeader\x12\x0f\n\x07\x64\x65leted\x18\x02 \x01(\x03\x12/\n\x08prev_kvs\x18\x03 \x03(\x0b\x32\x1d.bookkeeper.proto.kv.KeyValue\"\xdf\x01\n\tRequestOp\x12>\n\rrequest_range\x18\x01 \x01(\x0b\x32%.bookkeeper.proto.kv.rpc.RangeRequestH\x00\x12:\n\x0brequest_put\x18\x02 \x01(\x0b\x32#.bookkeeper.proto.kv.rpc.PutRequestH\x00\x12K\n\x14request_delete_range\x18\x03 \x01(\x0b\x32+.bookkeeper.proto.kv.rpc.DeleteRangeRequestH\x00\x42\t\n\x07request\"\xe7\x01\n\nResponseOp\x12@\n\x0eresponse_range\x18\x01 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RangeResponseH\x00\x12<\n\x0cresponse_put\x18\x02 \x01(\x0b\x32$.bookkeeper.proto.kv.rpc.PutResponseH\x00\x12M\n\x15response_delete_range\x18\x03 \x01(\x0b\x32,.bookkeeper.proto.kv.rpc.DeleteRangeResponseH\x00\x42\n\n\x08response\"\xfd\x02\n\x07\x43ompare\x12>\n\x06result\x18\x01 \x01(\x0e\x32..bookkeeper.proto.kv.rpc.Compare.CompareResult\x12>\n\x06target\x18\x02 \x01(\x0e\x32..bookkeeper.proto.kv.rpc.Compare.CompareTarget\x12\x0b\n\x03key\x18\x03 \x01(\x0c\x12\x11\n\x07version\x18\x04 \x01(\x03H\x00\x12\x19\n\x0f\x63reate_revision\x18\x05 \x01(\x03H\x00\x12\x16\n\x0cmod_revision\x18\x06 \x01(\x03H\x00\x12\x0f\n\x05value\x18\x07 \x01(\x0cH\x00\"@\n\rCompareResult\x12\t\n\x05\x45QUAL\x10\x00\x12\x0b\n\x07GREATER\x10\x01\x12\x08\n\x04LESS\x10\x02\x12\r\n\tNOT_EQUAL\x10\x03\"<\n\rCompareTarget\x12\x0b\n\x07VERSION\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\x07\n\x03MOD\x10\x02\x12\t\n\x05VALUE\x10\x03\x42\x0e\n\x0ctarget_union\"\xe1\x01\n\nTxnRequest\x12\x31\n\x07\x63ompare\x18\x01 \x03(\x0b\x32 .bookkeeper.proto.kv.rpc.Compare\x12\x33\n\x07success\x18\x02 \x03(\x0b\x32\".bookkeeper.proto.kv.rpc.RequestOp\x12\x33\n\x07\x66\x61ilure\x18\x03 \x03(\x0b\x32\".bookkeeper.proto.kv.rpc.RequestOp\x12\x36\n\x06header\x18\x63 \x01(\x0b\x32&.bookkeeper.proto.kv.rpc.RoutingHeader\"\x91\x01\n\x0bTxnResponse\x12\x37\n\x06header\x18\x01 \x01(\x0b\x32\'.bookkeeper.proto.kv.rpc.ResponseHeader\x12\x11\n\tsucceeded\x18\x02 \x01(\x08\x12\x36\n\tresponses\x18\x03 \x03(\x0b\x32#.bookkeeper.proto.kv.rpc.ResponseOp2\xdd\x03\n\x0cTableService\x12X\n\x05Range\x12%.bookkeeper.proto.kv.rpc.RangeRequest\x1a&.bookkeeper.proto.kv.rpc.RangeResponse\"\x00\x12R\n\x03Put\x12#.bookkeeper.proto.kv.rpc.PutRequest\x1a$.bookkeeper.proto.kv.rpc.PutResponse\"\x00\x12\x65\n\x06\x44\x65lete\x12+.bookkeeper.proto.kv.rpc.DeleteRangeRequest\x1a,.bookkeeper.proto.kv.rpc.DeleteRangeResponse\"\x00\x12R\n\x03Txn\x12#.bookkeeper.proto.kv.rpc.TxnRequest\x1a$.bookkeeper.proto.kv.rpc.TxnResponse\"\x00\x12\x64\n\tIncrement\x12).bookkeeper.proto.kv.rpc.IncrementRequest\x1a*.bookkeeper.proto.kv.rpc.IncrementResponse\"\x00\x42-\n)org.apache.bookkeeper.stream.proto.kv.rpcP\x01\x62\x06proto3')
  ,
  dependencies=[kv__pb2.DESCRIPTOR,storage__pb2.DESCRIPTOR,])



_RANGEREQUEST_SORTORDER = _descriptor.EnumDescriptor(
  name='SortOrder',
  full_name='bookkeeper.proto.kv.rpc.RangeRequest.SortOrder',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ASCEND', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DESCEND', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=719,
  serialized_end=765,
)
_sym_db.RegisterEnumDescriptor(_RANGEREQUEST_SORTORDER)

_RANGEREQUEST_SORTTARGET = _descriptor.EnumDescriptor(
  name='SortTarget',
  full_name='bookkeeper.proto.kv.rpc.RangeRequest.SortTarget',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='KEY', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VERSION', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MOD', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VALUE', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=767,
  serialized_end=833,
)
_sym_db.RegisterEnumDescriptor(_RANGEREQUEST_SORTTARGET)

_COMPARE_COMPARERESULT = _descriptor.EnumDescriptor(
  name='CompareResult',
  full_name='bookkeeper.proto.kv.rpc.Compare.CompareResult',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EQUAL', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GREATER', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LESS', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_EQUAL', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2502,
  serialized_end=2566,
)
_sym_db.RegisterEnumDescriptor(_COMPARE_COMPARERESULT)

_COMPARE_COMPARETARGET = _descriptor.EnumDescriptor(
  name='CompareTarget',
  full_name='bookkeeper.proto.kv.rpc.Compare.CompareTarget',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='VERSION', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MOD', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VALUE', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2568,
  serialized_end=2628,
)
_sym_db.RegisterEnumDescriptor(_COMPARE_COMPARETARGET)


_ROUTINGHEADER = _descriptor.Descriptor(
  name='RoutingHeader',
  full_name='bookkeeper.proto.kv.rpc.RoutingHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stream_id', full_name='bookkeeper.proto.kv.rpc.RoutingHeader.stream_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='range_id', full_name='bookkeeper.proto.kv.rpc.RoutingHeader.range_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='r_key', full_name='bookkeeper.proto.kv.rpc.RoutingHeader.r_key', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=133,
)


_RESPONSEHEADER = _descriptor.Descriptor(
  name='ResponseHeader',
  full_name='bookkeeper.proto.kv.rpc.ResponseHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='bookkeeper.proto.kv.rpc.ResponseHeader.code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='routing_header', full_name='bookkeeper.proto.kv.rpc.ResponseHeader.routing_header', index=1,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=136,
  serialized_end=268,
)


_RANGEREQUEST = _descriptor.Descriptor(
  name='RangeRequest',
  full_name='bookkeeper.proto.kv.rpc.RangeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bookkeeper.proto.kv.rpc.RangeRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='range_end', full_name='bookkeeper.proto.kv.rpc.RangeRequest.range_end', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='bookkeeper.proto.kv.rpc.RangeRequest.limit', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='bookkeeper.proto.kv.rpc.RangeRequest.revision', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sort_order', full_name='bookkeeper.proto.kv.rpc.RangeRequest.sort_order', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sort_target', full_name='bookkeeper.proto.kv.rpc.RangeRequest.sort_target', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serializable', full_name='bookkeeper.proto.kv.rpc.RangeRequest.serializable', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keys_only', full_name='bookkeeper.proto.kv.rpc.RangeRequest.keys_only', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count_only', full_name='bookkeeper.proto.kv.rpc.RangeRequest.count_only', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_mod_revision', full_name='bookkeeper.proto.kv.rpc.RangeRequest.min_mod_revision', index=9,
      number=10, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_mod_revision', full_name='bookkeeper.proto.kv.rpc.RangeRequest.max_mod_revision', index=10,
      number=11, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_create_revision', full_name='bookkeeper.proto.kv.rpc.RangeRequest.min_create_revision', index=11,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_create_revision', full_name='bookkeeper.proto.kv.rpc.RangeRequest.max_create_revision', index=12,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.RangeRequest.header', index=13,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RANGEREQUEST_SORTORDER,
    _RANGEREQUEST_SORTTARGET,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=271,
  serialized_end=833,
)


_RANGERESPONSE = _descriptor.Descriptor(
  name='RangeResponse',
  full_name='bookkeeper.proto.kv.rpc.RangeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.RangeResponse.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kvs', full_name='bookkeeper.proto.kv.rpc.RangeResponse.kvs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='more', full_name='bookkeeper.proto.kv.rpc.RangeResponse.more', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='count', full_name='bookkeeper.proto.kv.rpc.RangeResponse.count', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=836,
  serialized_end=981,
)


_PUTREQUEST = _descriptor.Descriptor(
  name='PutRequest',
  full_name='bookkeeper.proto.kv.rpc.PutRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bookkeeper.proto.kv.rpc.PutRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bookkeeper.proto.kv.rpc.PutRequest.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lease', full_name='bookkeeper.proto.kv.rpc.PutRequest.lease', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prev_kv', full_name='bookkeeper.proto.kv.rpc.PutRequest.prev_kv', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ignore_value', full_name='bookkeeper.proto.kv.rpc.PutRequest.ignore_value', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ignore_lease', full_name='bookkeeper.proto.kv.rpc.PutRequest.ignore_lease', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.PutRequest.header', index=6,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expected_version', full_name='bookkeeper.proto.kv.rpc.PutRequest.expected_version', index=7,
      number=100, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=984,
  serialized_end=1182,
)


_PUTRESPONSE = _descriptor.Descriptor(
  name='PutResponse',
  full_name='bookkeeper.proto.kv.rpc.PutResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.PutResponse.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prev_kv', full_name='bookkeeper.proto.kv.rpc.PutResponse.prev_kv', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1184,
  serialized_end=1302,
)


_INCREMENTREQUEST = _descriptor.Descriptor(
  name='IncrementRequest',
  full_name='bookkeeper.proto.kv.rpc.IncrementRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bookkeeper.proto.kv.rpc.IncrementRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='amount', full_name='bookkeeper.proto.kv.rpc.IncrementRequest.amount', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='get_total', full_name='bookkeeper.proto.kv.rpc.IncrementRequest.get_total', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.IncrementRequest.header', index=3,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1304,
  serialized_end=1426,
)


_INCREMENTRESPONSE = _descriptor.Descriptor(
  name='IncrementResponse',
  full_name='bookkeeper.proto.kv.rpc.IncrementResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.IncrementResponse.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='total_amount', full_name='bookkeeper.proto.kv.rpc.IncrementResponse.total_amount', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1428,
  serialized_end=1526,
)


_DELETERANGEREQUEST = _descriptor.Descriptor(
  name='DeleteRangeRequest',
  full_name='bookkeeper.proto.kv.rpc.DeleteRangeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bookkeeper.proto.kv.rpc.DeleteRangeRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='range_end', full_name='bookkeeper.proto.kv.rpc.DeleteRangeRequest.range_end', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prev_kv', full_name='bookkeeper.proto.kv.rpc.DeleteRangeRequest.prev_kv', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.DeleteRangeRequest.header', index=3,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1528,
  serialized_end=1653,
)


_DELETERANGERESPONSE = _descriptor.Descriptor(
  name='DeleteRangeResponse',
  full_name='bookkeeper.proto.kv.rpc.DeleteRangeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.DeleteRangeResponse.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deleted', full_name='bookkeeper.proto.kv.rpc.DeleteRangeResponse.deleted', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prev_kvs', full_name='bookkeeper.proto.kv.rpc.DeleteRangeResponse.prev_kvs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1656,
  serialized_end=1800,
)


_REQUESTOP = _descriptor.Descriptor(
  name='RequestOp',
  full_name='bookkeeper.proto.kv.rpc.RequestOp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='request_range', full_name='bookkeeper.proto.kv.rpc.RequestOp.request_range', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='request_put', full_name='bookkeeper.proto.kv.rpc.RequestOp.request_put', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='request_delete_range', full_name='bookkeeper.proto.kv.rpc.RequestOp.request_delete_range', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='request', full_name='bookkeeper.proto.kv.rpc.RequestOp.request',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1803,
  serialized_end=2026,
)


_RESPONSEOP = _descriptor.Descriptor(
  name='ResponseOp',
  full_name='bookkeeper.proto.kv.rpc.ResponseOp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='response_range', full_name='bookkeeper.proto.kv.rpc.ResponseOp.response_range', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='response_put', full_name='bookkeeper.proto.kv.rpc.ResponseOp.response_put', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='response_delete_range', full_name='bookkeeper.proto.kv.rpc.ResponseOp.response_delete_range', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='response', full_name='bookkeeper.proto.kv.rpc.ResponseOp.response',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=2029,
  serialized_end=2260,
)


_COMPARE = _descriptor.Descriptor(
  name='Compare',
  full_name='bookkeeper.proto.kv.rpc.Compare',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='bookkeeper.proto.kv.rpc.Compare.result', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='target', full_name='bookkeeper.proto.kv.rpc.Compare.target', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='bookkeeper.proto.kv.rpc.Compare.key', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='bookkeeper.proto.kv.rpc.Compare.version', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='create_revision', full_name='bookkeeper.proto.kv.rpc.Compare.create_revision', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mod_revision', full_name='bookkeeper.proto.kv.rpc.Compare.mod_revision', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bookkeeper.proto.kv.rpc.Compare.value', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _COMPARE_COMPARERESULT,
    _COMPARE_COMPARETARGET,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='target_union', full_name='bookkeeper.proto.kv.rpc.Compare.target_union',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=2263,
  serialized_end=2644,
)


_TXNREQUEST = _descriptor.Descriptor(
  name='TxnRequest',
  full_name='bookkeeper.proto.kv.rpc.TxnRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='compare', full_name='bookkeeper.proto.kv.rpc.TxnRequest.compare', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='success', full_name='bookkeeper.proto.kv.rpc.TxnRequest.success', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='failure', full_name='bookkeeper.proto.kv.rpc.TxnRequest.failure', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.TxnRequest.header', index=3,
      number=99, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2647,
  serialized_end=2872,
)


_TXNRESPONSE = _descriptor.Descriptor(
  name='TxnResponse',
  full_name='bookkeeper.proto.kv.rpc.TxnResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='bookkeeper.proto.kv.rpc.TxnResponse.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='succeeded', full_name='bookkeeper.proto.kv.rpc.TxnResponse.succeeded', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='responses', full_name='bookkeeper.proto.kv.rpc.TxnResponse.responses', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2875,
  serialized_end=3020,
)

_RESPONSEHEADER.fields_by_name['code'].enum_type = storage__pb2._STATUSCODE
_RESPONSEHEADER.fields_by_name['routing_header'].message_type = _ROUTINGHEADER
_RANGEREQUEST.fields_by_name['sort_order'].enum_type = _RANGEREQUEST_SORTORDER
_RANGEREQUEST.fields_by_name['sort_target'].enum_type = _RANGEREQUEST_SORTTARGET
_RANGEREQUEST.fields_by_name['header'].message_type = _ROUTINGHEADER
_RANGEREQUEST_SORTORDER.containing_type = _RANGEREQUEST
_RANGEREQUEST_SORTTARGET.containing_type = _RANGEREQUEST
_RANGERESPONSE.fields_by_name['header'].message_type = _RESPONSEHEADER
_RANGERESPONSE.fields_by_name['kvs'].message_type = kv__pb2._KEYVALUE
_PUTREQUEST.fields_by_name['header'].message_type = _ROUTINGHEADER
_PUTRESPONSE.fields_by_name['header'].message_type = _RESPONSEHEADER
_PUTRESPONSE.fields_by_name['prev_kv'].message_type = kv__pb2._KEYVALUE
_INCREMENTREQUEST.fields_by_name['header'].message_type = _ROUTINGHEADER
_INCREMENTRESPONSE.fields_by_name['header'].message_type = _RESPONSEHEADER
_DELETERANGEREQUEST.fields_by_name['header'].message_type = _ROUTINGHEADER
_DELETERANGERESPONSE.fields_by_name['header'].message_type = _RESPONSEHEADER
_DELETERANGERESPONSE.fields_by_name['prev_kvs'].message_type = kv__pb2._KEYVALUE
_REQUESTOP.fields_by_name['request_range'].message_type = _RANGEREQUEST
_REQUESTOP.fields_by_name['request_put'].message_type = _PUTREQUEST
_REQUESTOP.fields_by_name['request_delete_range'].message_type = _DELETERANGEREQUEST
_REQUESTOP.oneofs_by_name['request'].fields.append(
  _REQUESTOP.fields_by_name['request_range'])
_REQUESTOP.fields_by_name['request_range'].containing_oneof = _REQUESTOP.oneofs_by_name['request']
_REQUESTOP.oneofs_by_name['request'].fields.append(
  _REQUESTOP.fields_by_name['request_put'])
_REQUESTOP.fields_by_name['request_put'].containing_oneof = _REQUESTOP.oneofs_by_name['request']
_REQUESTOP.oneofs_by_name['request'].fields.append(
  _REQUESTOP.fields_by_name['request_delete_range'])
_REQUESTOP.fields_by_name['request_delete_range'].containing_oneof = _REQUESTOP.oneofs_by_name['request']
_RESPONSEOP.fields_by_name['response_range'].message_type = _RANGERESPONSE
_RESPONSEOP.fields_by_name['response_put'].message_type = _PUTRESPONSE
_RESPONSEOP.fields_by_name['response_delete_range'].message_type = _DELETERANGERESPONSE
_RESPONSEOP.oneofs_by_name['response'].fields.append(
  _RESPONSEOP.fields_by_name['response_range'])
_RESPONSEOP.fields_by_name['response_range'].containing_oneof = _RESPONSEOP.oneofs_by_name['response']
_RESPONSEOP.oneofs_by_name['response'].fields.append(
  _RESPONSEOP.fields_by_name['response_put'])
_RESPONSEOP.fields_by_name['response_put'].containing_oneof = _RESPONSEOP.oneofs_by_name['response']
_RESPONSEOP.oneofs_by_name['response'].fields.append(
  _RESPONSEOP.fields_by_name['response_delete_range'])
_RESPONSEOP.fields_by_name['response_delete_range'].containing_oneof = _RESPONSEOP.oneofs_by_name['response']
_COMPARE.fields_by_name['result'].enum_type = _COMPARE_COMPARERESULT
_COMPARE.fields_by_name['target'].enum_type = _COMPARE_COMPARETARGET
_COMPARE_COMPARERESULT.containing_type = _COMPARE
_COMPARE_COMPARETARGET.containing_type = _COMPARE
_COMPARE.oneofs_by_name['target_union'].fields.append(
  _COMPARE.fields_by_name['version'])
_COMPARE.fields_by_name['version'].containing_oneof = _COMPARE.oneofs_by_name['target_union']
_COMPARE.oneofs_by_name['target_union'].fields.append(
  _COMPARE.fields_by_name['create_revision'])
_COMPARE.fields_by_name['create_revision'].containing_oneof = _COMPARE.oneofs_by_name['target_union']
_COMPARE.oneofs_by_name['target_union'].fields.append(
  _COMPARE.fields_by_name['mod_revision'])
_COMPARE.fields_by_name['mod_revision'].containing_oneof = _COMPARE.oneofs_by_name['target_union']
_COMPARE.oneofs_by_name['target_union'].fields.append(
  _COMPARE.fields_by_name['value'])
_COMPARE.fields_by_name['value'].containing_oneof = _COMPARE.oneofs_by_name['target_union']
_TXNREQUEST.fields_by_name['compare'].message_type = _COMPARE
_TXNREQUEST.fields_by_name['success'].message_type = _REQUESTOP
_TXNREQUEST.fields_by_name['failure'].message_type = _REQUESTOP
_TXNREQUEST.fields_by_name['header'].message_type = _ROUTINGHEADER
_TXNRESPONSE.fields_by_name['header'].message_type = _RESPONSEHEADER
_TXNRESPONSE.fields_by_name['responses'].message_type = _RESPONSEOP
DESCRIPTOR.message_types_by_name['RoutingHeader'] = _ROUTINGHEADER
DESCRIPTOR.message_types_by_name['ResponseHeader'] = _RESPONSEHEADER
DESCRIPTOR.message_types_by_name['RangeRequest'] = _RANGEREQUEST
DESCRIPTOR.message_types_by_name['RangeResponse'] = _RANGERESPONSE
DESCRIPTOR.message_types_by_name['PutRequest'] = _PUTREQUEST
DESCRIPTOR.message_types_by_name['PutResponse'] = _PUTRESPONSE
DESCRIPTOR.message_types_by_name['IncrementRequest'] = _INCREMENTREQUEST
DESCRIPTOR.message_types_by_name['IncrementResponse'] = _INCREMENTRESPONSE
DESCRIPTOR.message_types_by_name['DeleteRangeRequest'] = _DELETERANGEREQUEST
DESCRIPTOR.message_types_by_name['DeleteRangeResponse'] = _DELETERANGERESPONSE
DESCRIPTOR.message_types_by_name['RequestOp'] = _REQUESTOP
DESCRIPTOR.message_types_by_name['ResponseOp'] = _RESPONSEOP
DESCRIPTOR.message_types_by_name['Compare'] = _COMPARE
DESCRIPTOR.message_types_by_name['TxnRequest'] = _TXNREQUEST
DESCRIPTOR.message_types_by_name['TxnResponse'] = _TXNRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RoutingHeader = _reflection.GeneratedProtocolMessageType('RoutingHeader', (_message.Message,), dict(
  DESCRIPTOR = _ROUTINGHEADER,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.RoutingHeader)
  ))
_sym_db.RegisterMessage(RoutingHeader)

ResponseHeader = _reflection.GeneratedProtocolMessageType('ResponseHeader', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEHEADER,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.ResponseHeader)
  ))
_sym_db.RegisterMessage(ResponseHeader)

RangeRequest = _reflection.GeneratedProtocolMessageType('RangeRequest', (_message.Message,), dict(
  DESCRIPTOR = _RANGEREQUEST,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.RangeRequest)
  ))
_sym_db.RegisterMessage(RangeRequest)

RangeResponse = _reflection.GeneratedProtocolMessageType('RangeResponse', (_message.Message,), dict(
  DESCRIPTOR = _RANGERESPONSE,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.RangeResponse)
  ))
_sym_db.RegisterMessage(RangeResponse)

PutRequest = _reflection.GeneratedProtocolMessageType('PutRequest', (_message.Message,), dict(
  DESCRIPTOR = _PUTREQUEST,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.PutRequest)
  ))
_sym_db.RegisterMessage(PutRequest)

PutResponse = _reflection.GeneratedProtocolMessageType('PutResponse', (_message.Message,), dict(
  DESCRIPTOR = _PUTRESPONSE,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.PutResponse)
  ))
_sym_db.RegisterMessage(PutResponse)

IncrementRequest = _reflection.GeneratedProtocolMessageType('IncrementRequest', (_message.Message,), dict(
  DESCRIPTOR = _INCREMENTREQUEST,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.IncrementRequest)
  ))
_sym_db.RegisterMessage(IncrementRequest)

IncrementResponse = _reflection.GeneratedProtocolMessageType('IncrementResponse', (_message.Message,), dict(
  DESCRIPTOR = _INCREMENTRESPONSE,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.IncrementResponse)
  ))
_sym_db.RegisterMessage(IncrementResponse)

DeleteRangeRequest = _reflection.GeneratedProtocolMessageType('DeleteRangeRequest', (_message.Message,), dict(
  DESCRIPTOR = _DELETERANGEREQUEST,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.DeleteRangeRequest)
  ))
_sym_db.RegisterMessage(DeleteRangeRequest)

DeleteRangeResponse = _reflection.GeneratedProtocolMessageType('DeleteRangeResponse', (_message.Message,), dict(
  DESCRIPTOR = _DELETERANGERESPONSE,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.DeleteRangeResponse)
  ))
_sym_db.RegisterMessage(DeleteRangeResponse)

RequestOp = _reflection.GeneratedProtocolMessageType('RequestOp', (_message.Message,), dict(
  DESCRIPTOR = _REQUESTOP,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.RequestOp)
  ))
_sym_db.RegisterMessage(RequestOp)

ResponseOp = _reflection.GeneratedProtocolMessageType('ResponseOp', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSEOP,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.ResponseOp)
  ))
_sym_db.RegisterMessage(ResponseOp)

Compare = _reflection.GeneratedProtocolMessageType('Compare', (_message.Message,), dict(
  DESCRIPTOR = _COMPARE,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.Compare)
  ))
_sym_db.RegisterMessage(Compare)

TxnRequest = _reflection.GeneratedProtocolMessageType('TxnRequest', (_message.Message,), dict(
  DESCRIPTOR = _TXNREQUEST,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.TxnRequest)
  ))
_sym_db.RegisterMessage(TxnRequest)

TxnResponse = _reflection.GeneratedProtocolMessageType('TxnResponse', (_message.Message,), dict(
  DESCRIPTOR = _TXNRESPONSE,
  __module__ = 'kv_rpc_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.rpc.TxnResponse)
  ))
_sym_db.RegisterMessage(TxnResponse)


DESCRIPTOR._options = None

_TABLESERVICE = _descriptor.ServiceDescriptor(
  name='TableService',
  full_name='bookkeeper.proto.kv.rpc.TableService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=3023,
  serialized_end=3500,
  methods=[
  _descriptor.MethodDescriptor(
    name='Range',
    full_name='bookkeeper.proto.kv.rpc.TableService.Range',
    index=0,
    containing_service=None,
    input_type=_RANGEREQUEST,
    output_type=_RANGERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Put',
    full_name='bookkeeper.proto.kv.rpc.TableService.Put',
    index=1,
    containing_service=None,
    input_type=_PUTREQUEST,
    output_type=_PUTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Delete',
    full_name='bookkeeper.proto.kv.rpc.TableService.Delete',
    index=2,
    containing_service=None,
    input_type=_DELETERANGEREQUEST,
    output_type=_DELETERANGERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Txn',
    full_name='bookkeeper.proto.kv.rpc.TableService.Txn',
    index=3,
    containing_service=None,
    input_type=_TXNREQUEST,
    output_type=_TXNRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Increment',
    full_name='bookkeeper.proto.kv.rpc.TableService.Increment',
    index=4,
    containing_service=None,
    input_type=_INCREMENTREQUEST,
    output_type=_INCREMENTRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TABLESERVICE)

DESCRIPTOR.services_by_name['TableService'] = _TABLESERVICE

# @@protoc_insertion_point(module_scope)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    # Try with C based implemenation if available
    import mmh3
except ImportError:
    # Fallback to pure python
    import pymmh3 as mmh3

__SEED__ = 383242705


class BytesHashRouter(object):
    """A router that computes hash values for keys using MurmurHash3"""

    def __init__(self):
        return

    def getRoutingKey(self, key):
        return mmh3.hash64(key, seed=__SEED__)[0]



# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from bookkeeper.common.constants import __DEFAULT_TABLE_CONF__
from bookkeeper.common.constants import __ROOT_RANGE_METADATA__
from bookkeeper.common.exceptions import from_root_range_rpc_response
from bookkeeper.common.exceptions import InternalServerError
from bookkeeper.common.exceptions import StreamExistsError
from bookkeeper.common.exceptions import StreamNotFoundError
from bookkeeper.common.method import wrap_method
from bookkeeper.common.retry import Retry
from bookkeeper.common.timeout import ExponentialTimeout
from bookkeeper.proto import storage_pb2
from bookkeeper.proto import stream_pb2


class Namespace(object):

    def __init__(self, root_range_service, namespace):
        self.__root_range_service__ = root_range_service
        self.__namespace__ = namespace
        self.__default_retry__ = Retry(deadline=60)
        self.__default_timeout__ = ExponentialTimeout()
        self.__create_with_retries__ =\
            wrap_method(self.__create_stream__, self.__default_retry__)
        self.__get_with_retries__ =\
            wrap_method(self.__get_stream__, self.__default_retry__)
        self.__delete_with_retries__ =\
            wrap_method(self.__delete_stream__, self.__default_retry__)

    def create(self, stream_name, stream_config=__DEFAULT_TABLE_CONF__):
        return self.__create_with_retries__(stream_name, stream_config)

    def __create_stream__(self, stream_name, stream_config):
        create_stream_req = storage_pb2.CreateStreamRequest(
            ns_name=self.__namespace__,
            name=stream_name,
            stream_conf=stream_config
        )
        create_stream_resp = self.__root_range_service__.CreateStream(
            request=create_stream_req,
            metadata=__ROOT_RANGE_METADATA__
        )
        try:
            create_stream_resp = from_root_range_rpc_response(create_stream_resp)
            return create_stream_resp.stream_props
        except InternalServerError as ise:
            # currently if a stream exists, it also throws
            # internal server error
            try:
                self.get(stream_name=stream_name)
                raise StreamExistsError("stream '%s' already exists at namespace '%s'"
                                        % (stream_name, self.__namespace__))
            except StreamNotFoundError:
                raise ise

    def get(self, stream_name):
        return self.__get_with_retries__(stream_name)

    def __get_stream__(self, stream_name):
        get_stream_req = storage_pb2.GetStreamRequest(
            stream_name=stream_pb2.StreamName(
                namespace_name=self.__namespace__,
                stream_name=stream_name
            )
        )
        get_stream_resp = self.__root_range_service__.GetStream(
            request=get_stream_req,
            metadata=__ROOT_RANGE_METADATA__
        )
        get_stream_resp = from_root_range_rpc_response(get_stream_resp)
        return get_stream_resp.stream_props

    def delete(self, stream_name):
        return self.__delete_with_retries__(stream_name)

    def __delete_stream__(self, stream_name):
        del_stream_req = storage_pb2.DeleteStreamRequest(
            ns_name=self.__namespace__,
            name=stream_name
        )
        del_stream_resp = self.__root_range_service__.DeleteStream(
            request=del_stream_req,
            metadata=__ROOT_RANGE_METADATA__
        )
        from_root_range_rpc_response(del_stream_resp)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import bookkeeper.proto.kv_rpc_pb2 as kv__rpc__pb2


class TableServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Range = channel.unary_unary(
        '/bookkeeper.proto.kv.rpc.TableService/Range',
        request_serializer=kv__rpc__pb2.RangeRequest.SerializeToString,
        response_deserializer=kv__rpc__pb2.RangeResponse.FromString,
        )
    self.Put = channel.unary_unary(
        '/bookkeeper.proto.kv.rpc.TableService/Put',
        request_serializer=kv__rpc__pb2.PutRequest.SerializeToString,
        response_deserializer=kv__rpc__pb2.PutResponse.FromString,
        )
    self.Delete = channel.unary_unary(
        '/bookkeeper.proto.kv.rpc.TableService/Delete',
        request_serializer=kv__rpc__pb2.DeleteRangeRequest.SerializeToString,
        response_deserializer=kv__rpc__pb2.DeleteRangeResponse.FromString,
        )
    self.Txn = channel.unary_unary(
        '/bookkeeper.proto.kv.rpc.TableService/Txn',
        request_serializer=kv__rpc__pb2.TxnRequest.SerializeToString,
        response_deserializer=kv__rpc__pb2.TxnResponse.FromString,
        )
    self.Increment = channel.unary_unary(
        '/bookkeeper.proto.kv.rpc.TableService/Increment',
        request_serializer=kv__rpc__pb2.IncrementRequest.SerializeToString,
        response_deserializer=kv__rpc__pb2.IncrementResponse.FromString,
        )


class TableServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def Range(self, request, context):
    """Range gets the keys in the range from the key-value store.
    NOT supported yet.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Put(self, request, context):
    """Put puts the given key into the key-value store.
    A put request increments the revision of the key-value store
    and generates one event in the event history.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Delete(self, request, context):
    """DeleteRange deletes the given range from the key-value store.
    A delete request increments the revision of the key-value store
    and generates a delete event in the event history for every deleted key.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Txn(self, request, context):
    """Txn processes multiple requests in a single transaction.
    A txn request increments the revision of the key-value store
    and generates events with the same revision for every completed request.
    It is not allowed to modify the same key several times within one txn.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Increment(self, request, context):
    """Increment increments the amount associated with the keys
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_TableServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Range': grpc.unary_unary_rpc_method_handler(
          servicer.Range,
          request_deserializer=kv__rpc__pb2.RangeRequest.FromString,
          response_serializer=kv__rpc__pb2.RangeResponse.SerializeToString,
      ),
      'Put': grpc.unary_unary_rpc_method_handler(
          servicer.Put,
          request_deserializer=kv__rpc__pb2.PutRequest.FromString,
          response_serializer=kv__rpc__pb2.PutResponse.SerializeToString,
      ),
      'Delete': grpc.unary_unary_rpc_method_handler(
          servicer.Delete,
          request_deserializer=kv__rpc__pb2.DeleteRangeRequest.FromString,
          response_serializer=kv__rpc__pb2.DeleteRangeResponse.SerializeToString,
      ),
      'Txn': grpc.unary_unary_rpc_method_handler(
          servicer.Txn,
          request_deserializer=kv__rpc__pb2.TxnRequest.FromString,
          response_serializer=kv__rpc__pb2.TxnResponse.SerializeToString,
      ),
      'Increment': grpc.unary_unary_rpc_method_handler(
          servicer.Increment,
          request_deserializer=kv__rpc__pb2.IncrementRequest.FromString,
          response_serializer=kv__rpc__pb2.IncrementResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'bookkeeper.proto.kv.rpc.TableService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
import collections
import sys

from google.protobuf import descriptor_pb2
from google.protobuf import duration_pb2
from google.protobuf import empty_pb2
from google.protobuf import field_mask_pb2
from google.protobuf import timestamp_pb2
from bookkeeper.proto import common_pb2
from bookkeeper.proto import kv_rpc_pb2
from bookkeeper.proto import stream_pb2
from bookkeeper.proto import storage_pb2

from bookkeeper.common.protobuf_helpers import get_messages


# Define the default values for storage client settings.
#
# This class is used when creating a bookkeeper client, and
# these settings can be altered to tweak client behavior.
# The defaults should be fine for most use cases.
StorageClientSettings = collections.namedtuple(
    'StorageClientSettings',
    ['service_uri'],
)
StorageClientSettings.__new__.__defaults__ = (
    "bk://localhost:4181",  # bookkeeper service uri
)

_shared_modules = [
    descriptor_pb2,
    duration_pb2,
    empty_pb2,
    field_mask_pb2,
    timestamp_pb2,
]

_local_modules = [
    common_pb2,
    kv_rpc_pb2,
    stream_pb2,
    storage_pb2
]

names = ['StorageClientSettings']

for module in _shared_modules:
    for name, message in get_messages(module).items():
        setattr(sys.modules[__name__], name, message)
        names.append(name)

for module in _local_modules:
    for name, message in get_messages(module).items():
        message.__module__ = 'bookkeeper.types'
        setattr(sys.modules[__name__], name, message)
        names.append(name)

__all__ = tuple(sorted(names))



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helpers for :mod:`datetime`."""

import calendar
import datetime
import re

import pytz


_UTC_EPOCH = datetime.datetime.utcfromtimestamp(0).replace(tzinfo=pytz.utc)
_RFC3339_MICROS = '%Y-%m-%dT%H:%M:%S.%fZ'
_RFC3339_NO_FRACTION = '%Y-%m-%dT%H:%M:%S'
# datetime.strptime cannot handle nanosecond precision:  parse w/ regex
_RFC3339_NANOS = re.compile(r"""
    (?P<no_fraction>
        \d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}  # YYYY-MM-DDTHH:MM:SS
    )
    (                                        # Optional decimal part
     \.                                      # decimal point
     (?P<nanos>\d{1,9})                      # nanoseconds, maybe truncated
    )?
    Z                                        # Zulu
""", re.VERBOSE)


def utcnow():
    """A :meth:`datetime.datetime.utcnow()` alias to allow mocking in tests."""
    return datetime.datetime.utcnow()


def to_milliseconds(value):
    """Convert a zone-aware datetime to milliseconds since the unix epoch.
    Args:
        value (datetime.datetime): The datetime to covert.
    Returns:
        int: Milliseconds since the unix epoch.
    """
    micros = to_microseconds(value)
    return micros // 1000


def from_microseconds(value):
    """Convert timestamp in microseconds since the unix epoch to datetime.
    Args:
        value (float): The timestamp to convert, in microseconds.
    Returns:
        datetime.datetime: The datetime object equivalent to the timestamp in
            UTC.
    """
    return _UTC_EPOCH + datetime.timedelta(microseconds=value)


def to_microseconds(value):
    """Convert a datetime to microseconds since the unix epoch.
    Args:
        value (datetime.datetime): The datetime to covert.
    Returns:
        int: Microseconds since the unix epoch.
    """
    if not value.tzinfo:
        value = value.replace(tzinfo=pytz.utc)
    # Regardless of what timezone is on the value, convert it to UTC.
    value = value.astimezone(pytz.utc)
    # Convert the datetime to a microsecond timestamp.
    return int(calendar.timegm(value.timetuple()) * 1e6) + value.microsecond


def from_iso8601_date(value):
    """Convert a ISO8601 date string to a date.
    Args:
        value (str): The ISO8601 date string.
    Returns:
        datetime.date: A date equivalent to the date string.
    """
    return datetime.datetime.strptime(value, '%Y-%m-%d').date()


def from_iso8601_time(value):
    """Convert a zoneless ISO8601 time string to a time.
    Args:
        value (str): The ISO8601 time string.
    Returns:
        datetime.time: A time equivalent to the time string.
    """
    return datetime.datetime.strptime(value, '%H:%M:%S').time()


def from_rfc3339(value):
    """Convert a microsecond-precision timestamp to datetime.
    Args:
        value (str): The RFC3339 string to convert.
    Returns:
        datetime.datetime: The datetime object equivalent to the timestamp in
            UTC.
    """
    return datetime.datetime.strptime(
        value, _RFC3339_MICROS).replace(tzinfo=pytz.utc)


def from_rfc3339_nanos(value):
    """Convert a nanosecond-precision timestamp to a native datetime.
    .. note::
        Python datetimes do not support nanosecond precision; this function
        therefore truncates such values to microseconds.
    Args:
        value (str): The RFC3339 string to convert.
    Returns:
        datetime.datetime: The datetime object equivalent to the timestamp in
            UTC.
    Raises:
        ValueError: If the timestamp does not match the RFC 3339
            regular expression.
    """
    with_nanos = _RFC3339_NANOS.match(value)

    if with_nanos is None:
        raise ValueError(
            'Timestamp: {!r}, does not match pattern: {!r}'.format(
                value, _RFC3339_NANOS.pattern))

    bare_seconds = datetime.datetime.strptime(
        with_nanos.group('no_fraction'), _RFC3339_NO_FRACTION)
    fraction = with_nanos.group('nanos')

    if fraction is None:
        micros = 0
    else:
        scale = 9 - len(fraction)
        nanos = int(fraction) * (10 ** scale)
        micros = nanos // 1000

    return bare_seconds.replace(microsecond=micros, tzinfo=pytz.utc)


def to_rfc3339(value, ignore_zone=True):
    """Convert a datetime to an RFC3339 timestamp string.
    Args:
        value (datetime.datetime):
            The datetime object to be converted to a string.
        ignore_zone (bool): If True, then the timezone (if any) of the
            datetime object is ignored and the datetime is treated as UTC.
    Returns:
        str: The RFC3339 formated string representing the datetime.
    """
    if not ignore_zone and value.tzinfo is not None:
        # Convert to UTC and remove the time zone info.
        value = value.replace(tzinfo=None) - value.utcoffset()

    return value.strftime(_RFC3339_MICROS)


class DatetimeWithNanoseconds(datetime.datetime):
    """Track nanosecond in addition to normal datetime attrs.
    Nanosecond can be passed only as a keyword argument.
    """
    __slots__ = ('_nanosecond',)

    # pylint: disable=arguments-differ
    def __new__(cls, *args, **kw):
        nanos = kw.pop('nanosecond', 0)
        if nanos > 0:
            if 'microsecond' in kw:
                raise TypeError(
                    "Specify only one of 'microsecond' or 'nanosecond'")
            kw['microsecond'] = nanos // 1000
        inst = datetime.datetime.__new__(cls, *args, **kw)
        inst._nanosecond = nanos or 0
        return inst
    # pylint: disable=arguments-differ

    @property
    def nanosecond(self):
        """Read-only: nanosecond precision."""
        return self._nanosecond

    def rfc3339(self):
        """Return an RFC 3339-compliant timestamp.
        Returns:
            (str): Timestamp string according to RFC 3339 spec.
        """
        if self._nanosecond == 0:
            return to_rfc3339(self)
        nanos = str(self._nanosecond).rstrip('0')
        return '{}.{}Z'.format(self.strftime(_RFC3339_NO_FRACTION), nanos)

    @classmethod
    def from_rfc3339(cls, stamp):
        """Parse RFC 3339-compliant timestamp, preserving nanoseconds.
        Args:
            stamp (str): RFC 3339 stamp, with up to nanosecond precision
        Returns:
            :class:`DatetimeWithNanoseconds`:
                an instance matching the timestamp string
        Raises:
            ValueError: if `stamp` does not match the expected format
        """
        with_nanos = _RFC3339_NANOS.match(stamp)
        if with_nanos is None:
            raise ValueError(
                'Timestamp: {}, does not match pattern: {}'.format(
                    stamp, _RFC3339_NANOS.pattern))
        bare = datetime.datetime.strptime(
            with_nanos.group('no_fraction'), _RFC3339_NO_FRACTION)
        fraction = with_nanos.group('nanos')
        if fraction is None:
            nanos = 0
        else:
            scale = 9 - len(fraction)
            nanos = int(fraction) * (10 ** scale)
        return cls(bare.year, bare.month, bare.day,
                   bare.hour, bare.minute, bare.second,
                   nanosecond=nanos, tzinfo=pytz.UTC)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import threading
import uuid

import bookkeeper.common.future
from bookkeeper.kv import exceptions


class Future(bookkeeper.common.future.Future):
    """Encapsulation of the asynchronous execution of an action.
    This object is returned from asychronous bookkeeper calls, and is the
    interface to determine the status of those calls.
    This object should not be created directly, but is returned by other
    methods in this library.
    Args:
        completed (Optional[Any]): An event, with the same interface as
            :class:`threading.Event`. This is provided so that callers
            with different concurrency models (e.g. ``threading`` or
            ``multiprocessing``) can supply an event that is compatible
            with that model. The ``wait()`` and ``set()`` methods will be
            used. If this argument is not provided, then a new
            :class:`threading.Event` will be created and used.
    """

    # This could be a sentinel object or None, but the sentinel object's ID
    # can change if the process is forked, and None has the possibility of
    # actually being a result.
    _SENTINEL = uuid.uuid4()

    def __init__(self, completed=None):
        self._result = self._SENTINEL
        self._exception = self._SENTINEL
        self._callbacks = []
        if completed is None:
            completed = threading.Event()
        self._completed = completed

    def cancel(self):
        """Actions in bookkeeper generally may not be canceled.
        This method always returns False.
        """
        return False

    def cancelled(self):
        """Actions in bookkeeper generally may not be canceled.
        This method always returns False.
        """
        return False

    def running(self):
        """Actions in bookkeeper generally may not be canceled.
        Returns:
            bool: ``True`` if this method has not yet completed, or
                ``False`` if it has completed.
        """
        if self.done():
            return False
        return True

    def done(self):
        """Return True the future is done, False otherwise.
        This still returns True in failure cases; checking :meth:`result` or
        :meth:`exception` is the canonical way to assess success or failure.
        """
        return (self._exception != self._SENTINEL or
                self._result != self._SENTINEL)

    def result(self, timeout=None):
        """Return the message ID, or raise an exception.
        This blocks until the message has successfully been published, and
        returns the message ID.
        Args:
            timeout (Union[int, float]): The number of seconds before this call
                times out and raises TimeoutError.
        Returns:
            str: The message ID.
        Raises:
            ~.pubsub_v1.TimeoutError: If the request times out.
            Exception: For undefined exceptions in the underlying
                call execution.
        """
        # Attempt to get the exception if there is one.
        # If there is not one, then we know everything worked, and we can
        # return an appropriate value.
        err = self.exception(timeout=timeout)
        if err is None:
            return self._result
        raise err

    def exception(self, timeout=None):
        """Return the exception raised by the call, if any.
        This blocks until the message has successfully been published, and
        returns the exception. If the call succeeded, return None.
        Args:
            timeout (Union[int, float]): The number of seconds before this call
                times out and raises TimeoutError.
        Raises:
            TimeoutError: If the request times out.
        Returns:
            Exception: The exception raised by the call, if any.
        """
        # Wait until the future is done.
        if not self._completed.wait(timeout=timeout):
            raise exceptions.TimeoutError('Timed out waiting for result.')

        # If the batch completed successfully, this should return None.
        if self._result != self._SENTINEL:
            return None

        # Okay, this batch had an error; this should return it.
        return self._exception

    def add_done_callback(self, fn):
        """Attach the provided callable to the future.
        The provided function is called, with this future as its only argument,
        when the future finishes running.
        """
        if self.done():
            return fn(self)
        self._callbacks.append(fn)

    def set_result(self, result):
        """Set the result of the future to the provided result.
        Args:
            result (Any): The result
        """
        # Sanity check: A future can only complete once.
        if self.done():
            raise RuntimeError('set_result can only be called once.')

        # Set the result and trigger the future.
        self._result = result
        self._trigger()

    def set_exception(self, exception):
        """Set the result of the future to the given exception.
        Args:
            exception (:exc:`Exception`): The exception raised.
        """
        # Sanity check: A future can only complete once.
        if self.done():
            raise RuntimeError('set_exception can only be called once.')

        # Set the exception and trigger the future.
        self._exception = exception
        self._trigger()

    def _trigger(self):
        """Trigger all callbacks registered to this Future.
        This method is called internally by the batch once the batch
        completes.
        Args:
            message_id (str): The message ID, as a string.
        """
        self._completed.set()
        for callback in self._callbacks:
            callback(self)



# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

import logging

from bookkeeper.common import util
from bookkeeper.common.exceptions import from_table_rpc_response
from bookkeeper.common.method import wrap_method
from bookkeeper.common.retry import Retry
from bookkeeper.common.timeout import ExponentialTimeout
from bookkeeper.proto import kv_rpc_pb2
from bookkeeper.proto.kv_rpc_pb2_grpc import TableServiceStub

__logger__ = logging.getLogger("bookkeeper.kv.Table")


class Table(object):

    def __init__(self, channel, stream_props):
        self.__table_service__ = TableServiceStub(channel=channel)
        self.__stream_props__ = stream_props
        self.__default_retry__ = Retry(deadline=60)
        self.__default_timeout__ = ExponentialTimeout()
        self.__put_with_retries__ =\
            wrap_method(self.__do_put__, self.__default_retry__)
        self.__get_with_retries__ =\
            wrap_method(self.__do_get__, self.__default_retry__)
        self.__del_with_retries__ =\
            wrap_method(self.__do_del__, self.__default_retry__)
        self.__incr_with_retries__ =\
            wrap_method(self.__do_incr__, self.__default_retry__)
        __logger__.info("initialized table instance with properties : %s",
                        stream_props)

    def __make_routing_metadata__(self, key):
        return [
            ('bk-rt-sid-bin',
             util.to_bytes(self.__stream_props__.stream_id, 8, "big")),
            ('bk-rt-key-bin',
             key)
        ]

    def __make_routing_header__(self, key):
        return kv_rpc_pb2.RoutingHeader(
            stream_id=self.__stream_props__.stream_id,
            r_key=key
        )

    def put_str(self, key_str, val_str):
        key = key_str.encode('utf-8')
        value = val_str.encode('utf-8')
        return self.put(key, value)

    def put(self, key, value):
        metadata = self.__make_routing_metadata__(key)
        header = self.__make_routing_header__(key)
        return self.__put_with_retries__(key, value, header, metadata)

    def __do_put__(self, key, value, routing_header, grpc_metadata):
        put_req = kv_rpc_pb2.PutRequest(
            key=key,
            value=value,
            header=routing_header
        )
        put_resp = self.__table_service__.Put(
            request=put_req,
            metadata=grpc_metadata
        )
        from_table_rpc_response(put_resp)

    def incr_str(self, key_str, amount):
        key = key_str.encode('utf-8')
        return self.incr(key, amount)

    def incr(self, key, amount):
        metadata = self.__make_routing_metadata__(key)
        header = self.__make_routing_header__(key)
        return self.__incr_with_retries__(key, amount, header, metadata)

    def __do_incr__(self, key, amount, routing_header, grpc_metadata):
        incr_req = kv_rpc_pb2.IncrementRequest(
            key=key,
            amount=amount,
            header=routing_header
        )
        incr_resp = self.__table_service__.Increment(
            request=incr_req,
            metadata=grpc_metadata
        )
        from_table_rpc_response(incr_resp)

    def get_str(self, key_str):
        key = key_str.encode('utf-8')
        return self.get(key)

    def get(self, key):
        metadata = self.__make_routing_metadata__(key)
        header = self.__make_routing_header__(key)
        return self.__get_with_retries__(key, header, metadata)

    def __do_get__(self, key, routing_header, grpc_metadata):
        get_req = kv_rpc_pb2.RangeRequest(
            key=key,
            header=routing_header
        )
        get_resp = self.__table_service__.Range(
            request=get_req,
            metadata=grpc_metadata
        )
        get_resp = from_table_rpc_response(get_resp)
        if get_resp.count == 0:
            return None
        else:
            return get_resp.kvs[0]

    def delete_str(self, key_str):
        key = key_str.encode('utf-8')
        return self.delete(key)

    def delete(self, key):
        metadata = self.__make_routing_metadata__(key)
        header = self.__make_routing_header__(key)
        return self.__del_with_retries__(key, header, metadata)

    def __do_del__(self, key, routing_header, grpc_metadata):
        del_req = kv_rpc_pb2.DeleteRangeRequest(
            key=key,
            header=routing_header
        )
        del_resp = self.__table_service__.Delete(
            request=del_req,
            metadata=grpc_metadata
        )
        del_resp = from_table_rpc_response(del_resp)
        return del_resp.deleted



# -*- coding: utf-8 -*-
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from bookkeeper.common import util
from bookkeeper.proto import stream_pb2

__ROOT_RANGE_ID__ = 0
__ROOT_RANGE_METADATA__ = [
        ('bk-rt-sc-id-bin', util.to_bytes(__ROOT_RANGE_ID__, 8, "big"))
]
__DEFAULT_STREAM_CONF__ = stream_pb2.StreamConfiguration(
        key_type=stream_pb2.RangeKeyType.values()[0],
        min_num_ranges=24,
        initial_num_ranges=4,
        split_policy=stream_pb2.SplitPolicy(
                type=stream_pb2.SplitPolicyType.values()[0],
                fixed_range_policy=stream_pb2.FixedRangeSplitPolicy(
                        num_ranges=2
                )
        ),
        rolling_policy=stream_pb2.SegmentRollingPolicy(
                size_policy=stream_pb2.SizeBasedSegmentRollingPolicy(
                        max_segment_size=128*1024*1024
                )
        ),
        retention_policy=stream_pb2.RetentionPolicy(
                time_policy=stream_pb2.TimeBasedRetentionPolicy(
                        retention_minutes=-1
                )
        ),
        storage_type=stream_pb2.StorageType.values()[0]
)
__DEFAULT_TABLE_CONF__ = stream_pb2.StreamConfiguration(
        key_type=stream_pb2.RangeKeyType.values()[0],
        min_num_ranges=24,
        initial_num_ranges=4,
        split_policy=stream_pb2.SplitPolicy(
                type=stream_pb2.SplitPolicyType.values()[0],
                fixed_range_policy=stream_pb2.FixedRangeSplitPolicy(
                        num_ranges=2
                )
        ),
        rolling_policy=stream_pb2.SegmentRollingPolicy(
                size_policy=stream_pb2.SizeBasedSegmentRollingPolicy(
                        max_segment_size=128*1024*1024
                )
        ),
        retention_policy=stream_pb2.RetentionPolicy(
                time_policy=stream_pb2.TimeBasedRetentionPolicy(
                        retention_minutes=-1
                )
        ),
        storage_type=stream_pb2.StorageType.values()[1]
)
__DEFAULT_NS_CONF__ = stream_pb2.NamespaceConfiguration(
        default_stream_conf=__DEFAULT_STREAM_CONF__
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cluster.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import bookkeeper.proto.common_pb2 as common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='cluster.proto',
  package='bookkeeper.proto.cluster',
  syntax='proto3',
  serialized_options=_b('\n*org.apache.bookkeeper.stream.proto.clusterP\001'),
  serialized_pb=_b('\n\rcluster.proto\x12\x18\x62ookkeeper.proto.cluster\x1a\x0c\x63ommon.proto\"*\n\x14ServerAssignmentData\x12\x12\n\ncontainers\x18\x01 \x03(\x03\"\xc6\x01\n\x15\x43lusterAssignmentData\x12M\n\x07servers\x18\x01 \x03(\x0b\x32<.bookkeeper.proto.cluster.ClusterAssignmentData.ServersEntry\x1a^\n\x0cServersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12=\n\x05value\x18\x02 \x01(\x0b\x32..bookkeeper.proto.cluster.ServerAssignmentData:\x02\x38\x01\"1\n\x0f\x43lusterMetadata\x12\x1e\n\x16num_storage_containers\x18\x01 \x01(\x03\x42.\n*org.apache.bookkeeper.stream.proto.clusterP\x01\x62\x06proto3')
  ,
  dependencies=[common__pb2.DESCRIPTOR,])




_SERVERASSIGNMENTDATA = _descriptor.Descriptor(
  name='ServerAssignmentData',
  full_name='bookkeeper.proto.cluster.ServerAssignmentData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='containers', full_name='bookkeeper.proto.cluster.ServerAssignmentData.containers', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=57,
  serialized_end=99,
)


_CLUSTERASSIGNMENTDATA_SERVERSENTRY = _descriptor.Descriptor(
  name='ServersEntry',
  full_name='bookkeeper.proto.cluster.ClusterAssignmentData.ServersEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='bookkeeper.proto.cluster.ClusterAssignmentData.ServersEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='bookkeeper.proto.cluster.ClusterAssignmentData.ServersEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=206,
  serialized_end=300,
)

_CLUSTERASSIGNMENTDATA = _descriptor.Descriptor(
  name='ClusterAssignmentData',
  full_name='bookkeeper.proto.cluster.ClusterAssignmentData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='servers', full_name='bookkeeper.proto.cluster.ClusterAssignmentData.servers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CLUSTERASSIGNMENTDATA_SERVERSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=102,
  serialized_end=300,
)


_CLUSTERMETADATA = _descriptor.Descriptor(
  name='ClusterMetadata',
  full_name='bookkeeper.proto.cluster.ClusterMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_storage_containers', full_name='bookkeeper.proto.cluster.ClusterMetadata.num_storage_containers', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=302,
  serialized_end=351,
)

_CLUSTERASSIGNMENTDATA_SERVERSENTRY.fields_by_name['value'].message_type = _SERVERASSIGNMENTDATA
_CLUSTERASSIGNMENTDATA_SERVERSENTRY.containing_type = _CLUSTERASSIGNMENTDATA
_CLUSTERASSIGNMENTDATA.fields_by_name['servers'].message_type = _CLUSTERASSIGNMENTDATA_SERVERSENTRY
DESCRIPTOR.message_types_by_name['ServerAssignmentData'] = _SERVERASSIGNMENTDATA
DESCRIPTOR.message_types_by_name['ClusterAssignmentData'] = _CLUSTERASSIGNMENTDATA
DESCRIPTOR.message_types_by_name['ClusterMetadata'] = _CLUSTERMETADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ServerAssignmentData = _reflection.GeneratedProtocolMessageType('ServerAssignmentData', (_message.Message,), dict(
  DESCRIPTOR = _SERVERASSIGNMENTDATA,
  __module__ = 'cluster_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.cluster.ServerAssignmentData)
  ))
_sym_db.RegisterMessage(ServerAssignmentData)

ClusterAssignmentData = _reflection.GeneratedProtocolMessageType('ClusterAssignmentData', (_message.Message,), dict(

  ServersEntry = _reflection.GeneratedProtocolMessageType('ServersEntry', (_message.Message,), dict(
    DESCRIPTOR = _CLUSTERASSIGNMENTDATA_SERVERSENTRY,
    __module__ = 'cluster_pb2'
    # @@protoc_insertion_point(class_scope:bookkeeper.proto.cluster.ClusterAssignmentData.ServersEntry)
    ))
  ,
  DESCRIPTOR = _CLUSTERASSIGNMENTDATA,
  __module__ = 'cluster_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.cluster.ClusterAssignmentData)
  ))
_sym_db.RegisterMessage(ClusterAssignmentData)
_sym_db.RegisterMessage(ClusterAssignmentData.ServersEntry)

ClusterMetadata = _reflection.GeneratedProtocolMessageType('ClusterMetadata', (_message.Message,), dict(
  DESCRIPTOR = _CLUSTERMETADATA,
  __module__ = 'cluster_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.cluster.ClusterMetadata)
  ))
_sym_db.RegisterMessage(ClusterMetadata)


DESCRIPTOR._options = None
_CLUSTERASSIGNMENTDATA_SERVERSENTRY._options = None
# @@protoc_insertion_point(module_scope)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from bookkeeper.common.util import __PYTHON3__
from bookkeeper.common.util import new_hostname_with_port

if __PYTHON3__:
    from urllib.parse import urlparse
else:
    from urlparse import urlparse


class ServiceURI(object):

    def __init__(self, service_uri):
        self.uri = urlparse(service_uri)
        self.service_name = self.uri.scheme
        self.service_user = self.uri.username
        self.service_path = self.uri.path
        if __PYTHON3__:
            self.service_hosts = list(map(lambda x: new_hostname_with_port(x), self.uri.netloc.split(',')))
        else:
            self.service_hosts = map(lambda x: new_hostname_with_port(x), self.uri.netloc.split(','))
        self.service_location = ','.join(self.service_hosts)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
import os

import setuptools

# Package metadata.

name = 'apache-bookkeeper-client'
description = 'Apache BookKeeper client library'
version = '4.15.4'
# Should be one of:
# 'Development Status :: 3 - Alpha'
# 'Development Status :: 4 - Beta'
# 'Development Status :: 5 - Production/Stable'
release_status = 'Development Status :: 3 - Alpha'
dependencies = [
    'protobuf>=3.0.0',
    'setuptools>=34.0.0',
    'six>=1.10.0',
    'pytz',
    'futures>=3.2.0;python_version<"3.2"',
    'grpcio<1.28,>=1.8.2',
    'pymmh3>=0.0.5'
]
extras = {
}

# Setup boilerplate below this line.

package_root = os.path.abspath(os.path.dirname(__file__))

readme_filename = os.path.join(package_root, 'README.rst')
with io.open(readme_filename, encoding='utf-8') as readme_file:
    readme = readme_file.read()

# Only include packages under the 'bookkeeper' namespace. Do not include tests,
# benchmarks, etc.
packages = [
    package for package in setuptools.find_packages()
    if package.startswith('bookkeeper')]

# Determine which namespaces are needed.
namespaces = ['bookkeeper']

setuptools.setup(
    name=name,
    version=version,
    description=description,
    long_description=readme,
    author='Apache BookKeeper',
    author_email='dev@bookkeeper.apache.org',
    license='Apache 2.0',
    url='https://github.com/apache/bookkeeper/tree/master/stream/clients/python',
    classifiers=[
        release_status,
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'Topic :: Internet',
    ],
    platforms='Posix; MacOS X; Windows',
    packages=packages,
    namespace_packages=namespaces,
    install_requires=dependencies,
    extras_require=extras,
    include_package_data=True,
    zip_safe=False,
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: kv_store.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import bookkeeper.proto.kv_pb2 as kv__pb2
import bookkeeper.proto.kv_rpc_pb2 as kv__rpc__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='kv_store.proto',
  package='bookkeeper.proto.kv.store',
  syntax='proto3',
  serialized_options=_b('\n+org.apache.bookkeeper.stream.proto.kv.storeP\001'),
  serialized_pb=_b('\n\x0ekv_store.proto\x12\x19\x62ookkeeper.proto.kv.store\x1a\x08kv.proto\x1a\x0ckv_rpc.proto\"\x83\x01\n\x07KeyMeta\x12\x17\n\x0f\x63reate_revision\x18\x01 \x01(\x03\x12\x14\n\x0cmod_revision\x18\x02 \x01(\x03\x12\x0f\n\x07version\x18\x03 \x01(\x03\x12\x38\n\nvalue_type\x18\x04 \x01(\x0e\x32$.bookkeeper.proto.kv.store.ValueType\"\x0c\n\nNopRequest\"\xbc\x02\n\x07\x43ommand\x12\x38\n\x07nop_req\x18\x01 \x01(\x0b\x32%.bookkeeper.proto.kv.store.NopRequestH\x00\x12\x36\n\x07put_req\x18\x02 \x01(\x0b\x32#.bookkeeper.proto.kv.rpc.PutRequestH\x00\x12\x41\n\ndelete_req\x18\x03 \x01(\x0b\x32+.bookkeeper.proto.kv.rpc.DeleteRangeRequestH\x00\x12\x36\n\x07txn_req\x18\x04 \x01(\x0b\x32#.bookkeeper.proto.kv.rpc.TxnRequestH\x00\x12=\n\x08incr_req\x18\x05 \x01(\x0b\x32).bookkeeper.proto.kv.rpc.IncrementRequestH\x00\x42\x05\n\x03req\"E\n\x12\x43heckpointMetadata\x12\r\n\x05\x66iles\x18\x01 \x03(\t\x12\x0c\n\x04txid\x18\x02 \x01(\x0c\x12\x12\n\ncreated_at\x18\x03 \x01(\x04*\"\n\tValueType\x12\t\n\x05\x42YTES\x10\x00\x12\n\n\x06NUMBER\x10\x01\x42/\n+org.apache.bookkeeper.stream.proto.kv.storeP\x01\x62\x06proto3')
  ,
  dependencies=[kv__pb2.DESCRIPTOR,kv__rpc__pb2.DESCRIPTOR,])

_VALUETYPE = _descriptor.EnumDescriptor(
  name='ValueType',
  full_name='bookkeeper.proto.kv.store.ValueType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='BYTES', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NUMBER', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=607,
  serialized_end=641,
)
_sym_db.RegisterEnumDescriptor(_VALUETYPE)

ValueType = enum_type_wrapper.EnumTypeWrapper(_VALUETYPE)
BYTES = 0
NUMBER = 1



_KEYMETA = _descriptor.Descriptor(
  name='KeyMeta',
  full_name='bookkeeper.proto.kv.store.KeyMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='create_revision', full_name='bookkeeper.proto.kv.store.KeyMeta.create_revision', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mod_revision', full_name='bookkeeper.proto.kv.store.KeyMeta.mod_revision', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='bookkeeper.proto.kv.store.KeyMeta.version', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_type', full_name='bookkeeper.proto.kv.store.KeyMeta.value_type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=70,
  serialized_end=201,
)


_NOPREQUEST = _descriptor.Descriptor(
  name='NopRequest',
  full_name='bookkeeper.proto.kv.store.NopRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=203,
  serialized_end=215,
)


_COMMAND = _descriptor.Descriptor(
  name='Command',
  full_name='bookkeeper.proto.kv.store.Command',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nop_req', full_name='bookkeeper.proto.kv.store.Command.nop_req', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='put_req', full_name='bookkeeper.proto.kv.store.Command.put_req', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delete_req', full_name='bookkeeper.proto.kv.store.Command.delete_req', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txn_req', full_name='bookkeeper.proto.kv.store.Command.txn_req', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='incr_req', full_name='bookkeeper.proto.kv.store.Command.incr_req', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='req', full_name='bookkeeper.proto.kv.store.Command.req',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=218,
  serialized_end=534,
)


_CHECKPOINTMETADATA = _descriptor.Descriptor(
  name='CheckpointMetadata',
  full_name='bookkeeper.proto.kv.store.CheckpointMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='files', full_name='bookkeeper.proto.kv.store.CheckpointMetadata.files', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txid', full_name='bookkeeper.proto.kv.store.CheckpointMetadata.txid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='bookkeeper.proto.kv.store.CheckpointMetadata.created_at', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=536,
  serialized_end=605,
)

_KEYMETA.fields_by_name['value_type'].enum_type = _VALUETYPE
_COMMAND.fields_by_name['nop_req'].message_type = _NOPREQUEST
_COMMAND.fields_by_name['put_req'].message_type = kv__rpc__pb2._PUTREQUEST
_COMMAND.fields_by_name['delete_req'].message_type = kv__rpc__pb2._DELETERANGEREQUEST
_COMMAND.fields_by_name['txn_req'].message_type = kv__rpc__pb2._TXNREQUEST
_COMMAND.fields_by_name['incr_req'].message_type = kv__rpc__pb2._INCREMENTREQUEST
_COMMAND.oneofs_by_name['req'].fields.append(
  _COMMAND.fields_by_name['nop_req'])
_COMMAND.fields_by_name['nop_req'].containing_oneof = _COMMAND.oneofs_by_name['req']
_COMMAND.oneofs_by_name['req'].fields.append(
  _COMMAND.fields_by_name['put_req'])
_COMMAND.fields_by_name['put_req'].containing_oneof = _COMMAND.oneofs_by_name['req']
_COMMAND.oneofs_by_name['req'].fields.append(
  _COMMAND.fields_by_name['delete_req'])
_COMMAND.fields_by_name['delete_req'].containing_oneof = _COMMAND.oneofs_by_name['req']
_COMMAND.oneofs_by_name['req'].fields.append(
  _COMMAND.fields_by_name['txn_req'])
_COMMAND.fields_by_name['txn_req'].containing_oneof = _COMMAND.oneofs_by_name['req']
_COMMAND.oneofs_by_name['req'].fields.append(
  _COMMAND.fields_by_name['incr_req'])
_COMMAND.fields_by_name['incr_req'].containing_oneof = _COMMAND.oneofs_by_name['req']
DESCRIPTOR.message_types_by_name['KeyMeta'] = _KEYMETA
DESCRIPTOR.message_types_by_name['NopRequest'] = _NOPREQUEST
DESCRIPTOR.message_types_by_name['Command'] = _COMMAND
DESCRIPTOR.message_types_by_name['CheckpointMetadata'] = _CHECKPOINTMETADATA
DESCRIPTOR.enum_types_by_name['ValueType'] = _VALUETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

KeyMeta = _reflection.GeneratedProtocolMessageType('KeyMeta', (_message.Message,), dict(
  DESCRIPTOR = _KEYMETA,
  __module__ = 'kv_store_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.store.KeyMeta)
  ))
_sym_db.RegisterMessage(KeyMeta)

NopRequest = _reflection.GeneratedProtocolMessageType('NopRequest', (_message.Message,), dict(
  DESCRIPTOR = _NOPREQUEST,
  __module__ = 'kv_store_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.store.NopRequest)
  ))
_sym_db.RegisterMessage(NopRequest)

Command = _reflection.GeneratedProtocolMessageType('Command', (_message.Message,), dict(
  DESCRIPTOR = _COMMAND,
  __module__ = 'kv_store_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.store.Command)
  ))
_sym_db.RegisterMessage(Command)

CheckpointMetadata = _reflection.GeneratedProtocolMessageType('CheckpointMetadata', (_message.Message,), dict(
  DESCRIPTOR = _CHECKPOINTMETADATA,
  __module__ = 'kv_store_pb2'
  # @@protoc_insertion_point(class_scope:bookkeeper.proto.kv.store.CheckpointMetadata)
  ))
_sym_db.RegisterMessage(CheckpointMetadata)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
